// Count elements in array
#define COUNT_OF(x) ((sizeof(x)/sizeof(0[x])) / ((size_t)(!(sizeof(x) % sizeof(0[x])))))

#include <Arduboy2.h>

Arduboy2 arduboy;

#define PLAYFIELD_WIDTH 10
#define PLAYFIELD_HEIGHT 20

struct Coordinate{
  byte x;
  byte y;  
};

class Cube {
  public:
    Coordinate blocks[4];
  
    Cube(){
      this->blocks[0].x = 0;
      this->blocks[0].y = 0;

      this->blocks[1].x = 1;
      this->blocks[1].y = 0;

      this->blocks[2].x = 0;
      this->blocks[2].y = 1;

      this->blocks[3].x = 1;
      this->blocks[3].y = 1;
    }
};

Cube selected;

bool playField[PLAYFIELD_WIDTH][PLAYFIELD_HEIGHT];
bool isStopped = false;

int rotateX(int x, int y){
  return 127-y;
}

int rotateY(int x, int y){
  return 63-x;
}

void drawPlayField(){
  int w = 52;
  int h = 100;
  //arduboy.drawLine(0,63,127,63);
  arduboy.drawLine(rotateX(0,0),rotateY(0,0),rotateX(0,h),rotateY(0,h)); // left wall
  arduboy.drawLine(rotateX(0,0),rotateY(0,0),rotateX(w,0),rotateY(w,0)); // bottom wall
  arduboy.drawLine(rotateX(w,0),rotateY(w,0),rotateX(w,h),rotateY(w,h)); // right wall
}

void drawBlocks(){
  for(int i=0; i<COUNT_OF(playField); i++){
    for(int j=0; j<COUNT_OF(playField[i]); j++){
      if(playField[i][j] == true){
        arduboy.drawRect(rotateX(5+i*5,5+j*5),rotateY(5+i*5,5+j*5),4,4);
      }
    }
  }   
}

void initializeSelected(){
  for(int i=0; i<COUNT_OF(selected.blocks); i++){
    selected.blocks[i].x = PLAYFIELD_WIDTH-5-selected.blocks[i].x;
    selected.blocks[i].y = PLAYFIELD_HEIGHT-1-selected.blocks[i].y;
    playField[selected.blocks[i].x][selected.blocks[i].y] = true;
  }
}

void dropSelected(){

      for(int i=0; i<4; i++){
        if(selected.blocks[i].y == 0 || selected.blocks[i].y-1 == true){
          isStopped = true;
          return;
        }
      }
  
      for(int i=0; i<4; i++){       
        playField[selected.blocks[i].x][selected.blocks[i].y] = false;
      }

      for(int i=0; i<4; i++){       
        selected.blocks[i].y--;
      }

      for(int i=0; i<4; i++){
        playField[selected.blocks[i].x][selected.blocks[i].y] = true;
      }
}

long prevMillis;

void setup() {
  Serial.begin(9600);
  arduboy.boot();

  arduboy.flipVertical(true);
  arduboy.flipHorizontal(true);

  arduboy.setFrameRate(15);

  Cube selected();
  initializeSelected();
}


void loop() {
  // pause render until it's time for the next frame
  if (!(arduboy.nextFrame()))
    return;  
  
  arduboy.clear();

  drawPlayField();  

  if(arduboy.pressed(LEFT_BUTTON)){
    bool isMin = false;
    for(int i=0; i<4; i++){
      if(selected.blocks[i].x == 0){
        isMin = true;
        break;
      }
    }
    if(isMin == false){    
      for(int i=0; i<4; i++){
        playField[selected.blocks[i].x][selected.blocks[i].y] = false;
        selected.blocks[i].x--;
      }
      for(int i=0; i<4; i++){
        playField[selected.blocks[i].x][selected.blocks[i].y] = true;
      }
    }
  }
  else if(arduboy.pressed(RIGHT_BUTTON)){
    bool isMax = false;
    for(int i=0; i<4; i++){
      if(selected.blocks[i].x == PLAYFIELD_WIDTH-1){
        isMax = true;
        break;
      }
    }
    if(isMax == false){    
      for(int i=0; i<4; i++){
        playField[selected.blocks[i].x][selected.blocks[i].y] = false;
        selected.blocks[i].x++;
      }
      for(int i=0; i<4; i++){
        playField[selected.blocks[i].x][selected.blocks[i].y] = true;
      }
    }
  }  

  if(millis() > prevMillis + 100){    
    if(isStopped){
      Cube selected();
      initializeSelected();
      isStopped = false;
    }
    else{
      dropSelected();
    }
    prevMillis = millis();
  }

  drawBlocks();
    
  arduboy.display();
}
