// Count elements in array
#define COUNT_OF(x) ((sizeof(x)/sizeof(0[x])) / ((size_t)(!(sizeof(x) % sizeof(0[x])))))
#define PLAYFIELD_WIDTH 10
#define PLAYFIELD_HEIGHT 20

#include <Arduboy2.h>

Arduboy2 arduboy;

int rotateX(int x, int y){
  return y;
}

int rotateY(int x, int y){
  return x;
}

struct Coordinate{
  byte x, y;
};

class Tetrimino{
  public:
    Tetrimino();
    Tetrimino(byte x0, byte y0, byte x1, byte y1, byte x2, byte y2, byte x3, byte y3);
    Coordinate blocks[4];
    void initialize();    
    void clear();
    void appear();
    void drop();
    void move(int direction);
    bool shouldStop();
    int getMinX();
    int getMinY();
    int getMaxX();
    int getMaxY();
    bool containsBlock(byte x, byte y);
};

class PlayField{
  public:
    static bool blocks[PLAYFIELD_WIDTH][PLAYFIELD_HEIGHT];

    static void drawContainer(){
      int w = 52;
      int h = 100;
      arduboy.drawLine(rotateX(0,0),rotateY(0,0),rotateX(0,h),rotateY(0,h)); // left wall
      arduboy.drawLine(rotateX(0,0),rotateY(0,0),rotateX(w,0),rotateY(w,0)); // bottom wall
      arduboy.drawLine(rotateX(w,0),rotateY(w,0),rotateX(w,h),rotateY(w,h)); // right wall
    }

    static void drawBlocks(){
      byte offset = 2;
      for(int i=0; i<PLAYFIELD_WIDTH; i++){
        for(int j=0; j<PLAYFIELD_HEIGHT; j++){
          if(blocks[i][j] == true){
            arduboy.drawRect(rotateX(offset+i*5,offset+j*5),rotateY(offset+i*5,offset+j*5),4,4);
          }
        }
      }  
    }

    static bool hasFullLine(byte y){      
      for(int i=0; i<PLAYFIELD_WIDTH; i++){
        if(PlayField::blocks[i][y] == false){
          return false;
        }
      }
      return true;
    }

    static void clearLine(Tetrimino selected, byte y){      
        for(int i=0; i<PLAYFIELD_WIDTH; i++){                  
            PlayField::blocks[i][y] = PlayField::blocks[i][y+1];                       
        }
    }

};

static bool PlayField::blocks[PLAYFIELD_WIDTH][PLAYFIELD_HEIGHT];

Tetrimino::Tetrimino(){  
}

Tetrimino::Tetrimino(byte x0, byte y0, byte x1, byte y1, byte x2, byte y2, byte x3, byte y3){
  blocks[0].x = x0;
  blocks[0].y = y0;      
  
  blocks[1].x = x1;
  blocks[1].y = y1;
  
  blocks[2].x = x2;
  blocks[2].y = y2;

  blocks[3].x = x3;
  blocks[3].y = y3;
}

void Tetrimino::initialize(){
  for(int i=0; i<COUNT_OF(blocks); i++){
    blocks[i].x = PLAYFIELD_WIDTH - 5 - blocks[i].x;
    blocks[i].y = PLAYFIELD_HEIGHT - 1 - blocks[i].y;
    PlayField::blocks[blocks[i].x][blocks[i].y] = true;
  }
}

void Tetrimino::clear(){
  for(int i=0; i<COUNT_OF(blocks); i++){
    PlayField::blocks[blocks[i].x][blocks[i].y] = false;        
  }
}

void Tetrimino::appear(){
  for(int i=0; i<COUNT_OF(blocks); i++){
    PlayField::blocks[blocks[i].x][blocks[i].y] = true;
  }
}

void Tetrimino::drop(){     
  clear();
  for(int i=0; i<COUNT_OF(blocks); i++){
    PlayField::blocks[blocks[i].x][--blocks[i].y];
  }
  appear();
}

void Tetrimino::move(int direction){
  if(direction == LEFT_BUTTON && blocks[getMinX()].x > 0){
    clear();
    for(int i=0; i<COUNT_OF(blocks); i++){
      --blocks[i].x;
    }
    appear();
  }
  else if (direction == RIGHT_BUTTON && blocks[getMaxX()].x < PLAYFIELD_WIDTH-1){
   clear();
   for(int i=0; i<COUNT_OF(blocks); i++){
      ++blocks[i].x;
   }
   appear();
  }
}

bool Tetrimino::shouldStop(){
  for(int i=0; i<COUNT_OF(blocks); i++){
    if(blocks[i].y == 0)
      return true;
  }
  
  byte minYBlock = getMinY();
  if(PlayField::blocks[blocks[minYBlock].x][blocks[minYBlock].y-1] == true){
    return true;
  }
  
  return false;
}

int Tetrimino::getMinX(){
    int minX = 0;
    for(int i=0; i<COUNT_OF(blocks); i++){
      if(blocks[i].x < blocks[minX].x){
        minX = i;
      }
    }
    return minX;
}

int Tetrimino::getMinY(){
    int minY = 0;
    for(int i=0; i<COUNT_OF(blocks); i++){
      if(blocks[i].y < blocks[minY].y){
        minY = i;
      }
    }
    return minY;
}

int Tetrimino::getMaxX(){
    int maxX = 0;
    for(int i=0; i<COUNT_OF(blocks); i++){
      if(blocks[i].x > blocks[maxX].x){
        maxX = i;
      }
    }
    return maxX;
}

bool Tetrimino::containsBlock(byte x, byte y){
  for(int i=0; i<COUNT_OF(blocks); i++){
    if(blocks[i].x == x && blocks[i].y == y)
      return true;  
  }
  return false;
}


class Box: public Tetrimino{
  public:
    Box(): Tetrimino(0,0,0,1,1,0,1,1){};
};

class Long : public Tetrimino{
  public:
    Long() : Tetrimino(0,0,0,1,0,2,0,3){};
};

class Tee : public Tetrimino{
  public:
    Tee() : Tetrimino(0,1,1,1,2,1,1,0){};  
};

class Saw : public Tetrimino{
  public:
    Saw() : Tetrimino(0,0,1,0,1,1,2,1){};  
};

class Zaw : public Tetrimino{
  public:
    Zaw() : Tetrimino(0,1,1,1,1,0,2,0){};
};

class Jay : public Tetrimino{
  public:
    Jay() : Tetrimino(0,0,1,0,2,0,0,1){};
};

class Elle : public Tetrimino{
  public:
    Elle() : Tetrimino(0,0,1,0,2,0,2,1){};    
};

Tetrimino selected;
long prevMillis;

void setup() {
  arduboy.boot();
  arduboy.setFrameRate(15); 
  selected = Box();
  //selected = Long();
  //selected = Tee();
  //selected = Saw();
  //selected = Zaw();
  //selected = Jay();
  //selected = Elle();
  selected.initialize();
}

void loop() {
  if (!(arduboy.nextFrame()))
    return;  
  
  arduboy.clear();

  PlayField::drawContainer();  

  if(!selected.shouldStop()){
    if(millis() > prevMillis + 100){
      selected.drop();
      if(arduboy.pressed(LEFT_BUTTON)){ selected.move(LEFT_BUTTON); }
      else if (arduboy.pressed(RIGHT_BUTTON)){ selected.move(RIGHT_BUTTON); }
      prevMillis = millis();
    }
  }
  else{
    selected = Box();
    selected.initialize();
    
    for(int i=PLAYFIELD_HEIGHT-3; i>-1; i--){
      if(PlayField::hasFullLine(i)){
        PlayField::clearLine(selected, i);
      }
    }
  }

  PlayField::drawBlocks(); 
  
  arduboy.display();

}
