#define COUNT_OF(x) ((sizeof(x)/sizeof(0[x])) / ((size_t)(!(sizeof(x) % sizeof(0[x])))))
#define PLAYFIELD_WIDTH 10
#define PLAYFIELD_HEIGHT 20
#define BLOCK_SIZE 4

enum Pieces{BOX, LONG, TEE, SAW, ZAW, JAY, ELLE};

#include <Arduboy2.h>

Arduboy2 arduboy;

bool gameOver = false;
bool gameStarted = true;
int linesCleared = 0;
int score = 0;
long prevMillis;

class Coordinate{
  public:
    byte x, y;
    Coordinate(){}
    Coordinate(byte x, byte y){
      this->x = x;
      this->y = y;
    }
};

class PlayField{
  public:
    static bool blocks[PLAYFIELD_WIDTH][PLAYFIELD_HEIGHT];
    
    static void drawContainer(){
      int w = 2 + (BLOCK_SIZE+1) * PLAYFIELD_WIDTH;
      int h = (BLOCK_SIZE+1) * PLAYFIELD_HEIGHT;
      arduboy.drawLine(0,0,h,0); // left wall
      arduboy.drawLine(0,0,0,w); // bottom wall
      arduboy.drawLine(0,w,h,w); // right wall
    }

    static void drawBlocks(){
      byte padding = 2;
      for(int i=0; i<PLAYFIELD_WIDTH; i++){
        for(int j=0; j<PLAYFIELD_HEIGHT; j++){
          if(blocks[i][j] == true){
            arduboy.drawRect(padding+j*(BLOCK_SIZE+1),padding+i*(BLOCK_SIZE+1),BLOCK_SIZE,BLOCK_SIZE); // rotated x and y axes for vertical screen
          }
        }
      }  
    }

    static void clear(){
      for(int i=0; i<PLAYFIELD_WIDTH; i++){
        for(int j=0; j<PLAYFIELD_HEIGHT; j++){
          PlayField::blocks[i][j] = false;
        }
      }
    }

    static bool findLine(byte y){
      for(int i=0; i < PLAYFIELD_WIDTH; i++){
        if(PlayField::blocks[i][y] == false){
          return false;
        }
      }
      return true;
    }

    static void clearLine(byte y){
      for(int i=0; i < PLAYFIELD_WIDTH; i++){
        for(int j=y; j < PLAYFIELD_HEIGHT-1; j++){
          PlayField::blocks[i][j] = PlayField::blocks[i][j+1];
        }
      }
    }

    static void findAndClearLines(){
      for(int i=0; i<PLAYFIELD_HEIGHT-1; i++){
        if(PlayField::findLine(i)){
          PlayField::clearLine(i--); // lines shift downwards when cleared so index must be decremented to check line that was above
          ++linesCleared;
        }
      }
    }
};

static bool PlayField::blocks[PLAYFIELD_WIDTH][PLAYFIELD_HEIGHT];

class Tetrimino{
  public:
    Pieces piece;
    Coordinate blocks[4];
    bool stopped = false;
    int angle = 0;

    Tetrimino(){}
    
    Tetrimino(Pieces piece){
      this->piece = piece;
      switch(piece){
        case BOX: setBlocks(0,0,0,1,1,0,1,1); break;
        case LONG: setBlocks(0,0,0,1,0,2,0,3); break;        
        case TEE: setBlocks(0,1,1,1,2,1,1,0); break;
        case SAW: setBlocks(0,0,1,0,1,1,2,1); break;
        case ZAW: setBlocks(0,1,1,1,1,0,2,0); break;
        case JAY: setBlocks(0,0,1,0,2,0,0,1); break;
        case ELLE: setBlocks(0,0,1,0,2,0,2,1); break;
      }
    }
    
    void setBlocks(byte x0, byte y0, byte x1, byte y1, byte x2, byte y2, byte x3, byte y3){      
      blocks[0].x = x0;
      blocks[0].y = y0;      
      
      blocks[1].x = x1;
      blocks[1].y = y1;
      
      blocks[2].x = x2;
      blocks[2].y = y2;
    
      blocks[3].x = x3;
      blocks[3].y = y3;
    }

    bool moveToTop(){
      for(int i=0; i<COUNT_OF(blocks); i++){
        blocks[i].x = PLAYFIELD_WIDTH - PLAYFIELD_WIDTH/2 - blocks[i].x;
        blocks[i].y = PLAYFIELD_HEIGHT - 1 - blocks[i].y;
        if(PlayField::blocks[blocks[i].x][blocks[i].y] == true){
          return false;
        }
        PlayField::blocks[blocks[i].x][blocks[i].y] = true;
      }
      return true;
    }

    void clear(){
      for(int i=0; i<COUNT_OF(blocks); i++){
        PlayField::blocks[blocks[i].x][blocks[i].y] = false;
      }
    }

    void appear(){
      for(int i=0; i<COUNT_OF(blocks); i++){
        PlayField::blocks[blocks[i].x][blocks[i].y] = true;
      }
    }

    void drop(){
      if(shouldStop()){
        stopped = true;
        return;
      }
      clear();      
      for(int i=0; i<COUNT_OF(blocks); i++){
        PlayField::blocks[blocks[i].x][--blocks[i].y] = true;
      } 
      appear();
    }

    byte minY(){ // which of the 4 blocks is at the lowest position?
      int min = COUNT_OF(blocks)-1;
      for(int i=0; i<COUNT_OF(blocks); i++){
        if(blocks[i].y < blocks[min].y){
          min = i;
        }
      }
      return min;
    }

    bool shouldStop(){
      if(blocks[minY()].y == 0){ // hit the bottom
        return true;
      }
      for(int i=0; i<COUNT_OF(blocks); i++){
        if(PlayField::blocks[blocks[i].x][blocks[i].y-1] == true && isSelf(blocks[i].x,blocks[i].y-1) == false){
          return true;
        }
      }
      return false;
    }

    bool isSelf(byte x, byte y){
      for(int i=0; i<COUNT_OF(blocks); i++){
        if(blocks[i].x == x && blocks[i].y == y){
          return true;
        }  
      }  
      return false;
    }

    byte minX(){
      int min = COUNT_OF(blocks)-1;
      for(int i=0; i<COUNT_OF(blocks); i++){
        if(blocks[i].x < blocks[min].x){
          min = i;
        }
      }
      return min;
    }

    bool canMoveLeft(){
      if(blocks[minX()].x == 0){
        return false;
      }

      for(int i=0; i<COUNT_OF(blocks); i++){
        if(isSelf(blocks[i].x-1, blocks[i].y) == false && PlayField::blocks[blocks[i].x-1][blocks[i].y] == true){
          return false;
        }
      }
      
      return true;
    }

    void moveLeft(){
      if(canMoveLeft()){
        clear();
        for(int i=0; i<COUNT_OF(blocks); i++){
          PlayField::blocks[--blocks[i].x][blocks[i].y] = true;
        }         
        appear();
      }
    }

    byte maxX(){
      int max = COUNT_OF(blocks)-1;
      for(int i=0; i<COUNT_OF(blocks); i++){
        if(blocks[i].x > blocks[max].x){
          max = i;
        }  
      }  
      return max;
    }

    bool canMoveRight(){
      if(blocks[maxX()].x == PLAYFIELD_WIDTH-1){
        return false;
      }

      for(int i=0; i<COUNT_OF(blocks); i++){
        if(PlayField::blocks[blocks[i].x+1][blocks[i].y] == true && isSelf(blocks[i].x+1, blocks[i].y) == false){
          return false;
        }
      }
      return true;
    }    

    void moveRight(){
      if(canMoveRight()){
        clear();
        for(int i=0; i<COUNT_OF(blocks); i++){
          PlayField::blocks[++blocks[i].x][blocks[i].y] = true;
        }         
        appear();
      }      
    }

    bool canShiftBlock(byte offsetX0, byte offsetY0, byte offsetX1, byte offsetY1, byte offsetX2, byte offsetY2, byte offsetX3, byte offsetY3){
      Tetrimino rotated;
      rotated.setBlocks(
        blocks[0].x + offsetX0, 
        blocks[0].y + offsetY0, 
        blocks[1].x + offsetX1, 
        blocks[1].y + offsetY1, 
        blocks[2].x + offsetX2, 
        blocks[2].y + offsetY2, 
        blocks[3].x + offsetX3, 
        blocks[3].y + offsetY3
      );

      for(int i=0; i<COUNT_OF(rotated.blocks); i++){
        if(rotated.blocks[i].x < 0 || rotated.blocks[i].x > PLAYFIELD_WIDTH-1){
          return false;
        }
      }

      for(int i=0; i<COUNT_OF(rotated.blocks); i++){
        if(PlayField::blocks[rotated.blocks[i].x][rotated.blocks[i].y] == true && rotated.isSelf(blocks[i].x, blocks[i].y) == false){
          return false;          
        }
      }

      return true;
    }

    void shiftBlock(byte offsetX0, byte offsetY0, byte offsetX1, byte offsetY1, byte offsetX2, byte offsetY2, byte offsetX3, byte offsetY3){
      if(canShiftBlock(offsetX0, offsetY0, offsetX1, offsetY1, offsetX2, offsetY2, offsetX3, offsetY3) == false){
        return;
      }
      blocks[0].x += offsetX0;
      blocks[0].y += offsetY0;
      blocks[1].x += offsetX1;
      blocks[1].y += offsetY1;
      blocks[2].x += offsetX2;
      blocks[2].y += offsetY2;
      blocks[3].x += offsetX3;
      blocks[3].y += offsetY3;
    }

    void swap(Coordinate a, Coordinate b){
      Coordinate temp = a;
      a = b;
      b = temp;
    }

    bool rotate(){
      clear();
      if(this->piece == BOX){
        return;
      }
      else if (this->piece == LONG){        
        if (angle == 0 || angle == 180){
          shiftBlock(-1,-1,0,0,1,1,2,2);
        }
        else if (angle == 90 || angle == 270){
          shiftBlock(1,1,0,0,-1,-1,-2,-2);
        }
      }
      else if (this->piece == SAW){
        if (angle == 0 || angle == 180){
          shiftBlock(0,0,0,0,1,0,1,2);
        }  
        else if (angle == 90 || angle == 270){
          shiftBlock(0,0,0,0,-1,0,-1,-2);
        }
      }
      else if (this->piece == ZAW){
        if (angle == 0 || angle == 180){
          shiftBlock(0,0,1,1,0,0,1,1);
        }
        else if (angle == 90 || angle == 270){
          shiftBlock(0,0,-1,-1,0,0,-1,-1);
        }
      }
      else if (this->piece == TEE){
        if (angle == 0){
          shiftBlock(0,0,0,0,1,-1,0,0);
        }
        else if(angle == 90){
          shiftBlock(0,0,0,0,0,0,-1,-1);
        }
        else if(angle == 180){
          shiftBlock(-1,1,0,0,0,0,0,0);
        }
        else if (angle == 270){
          shiftBlock(0,0,0,0,1,1,0,0);          
        }
      }
      else if (this->piece == JAY){
        
      }
      else if (this->piece == ELLE){
        
      }
      
      angle = (angle + 90) % 360;
      clear();
      for(int i=0; i<COUNT_OF(blocks); i++){
        PlayField::blocks[blocks[i].x][blocks[i].y] = true;
      }         
      appear();
    }
};

Tetrimino randomPiece(){
  return Tetrimino(TEE);
  //return Tetrimino(ZAW);
  //return Tetrimino(SAW);
  //return Tetrimino(LONG);
  //return Tetrimino(random(0,7));
}

Tetrimino selected;

void startGame(){
    PlayField::clear();
    selected = randomPiece();
    selected.moveToTop();
    gameStarted = true;
    gameOver = false;
    linesCleared = 0;
    score = 0;
}

void endGame(){
    arduboy.println("Game Over");
    arduboy.print("Lines: ");
    arduboy.println(linesCleared);
    arduboy.display();  
}

void setup() {
  arduboy.boot();
  arduboy.setFrameRate(60);
  arduboy.initRandomSeed();
}

void loop() {
  if (!(arduboy.nextFrame()))
    return;

  if(gameStarted == false && arduboy.pressed(A_BUTTON)){
    startGame();
  }

  arduboy.clear();

  PlayField::drawContainer();
    
  if(!gameOver){
    if(millis() > prevMillis + 100){
      
      if(millis() < 300){
        selected.drop();        
      }
      if(selected.stopped){
        PlayField::findAndClearLines();
        selected = randomPiece();
        if(selected.moveToTop() == false){
          gameOver = true;
          gameStarted = false;          
        }
      }
      else if (arduboy.pressed(LEFT_BUTTON)){
        selected.moveLeft();
      }
      else if(arduboy.pressed(RIGHT_BUTTON)){
        selected.moveRight();
      }
      else if (arduboy.pressed(DOWN_BUTTON)){
        selected.drop();
      }
      else if (arduboy.pressed(A_BUTTON)){
        selected.rotate();
      }
      prevMillis = millis();
    }
  }
  else{
    endGame();
    return;
  }
  
  PlayField::drawBlocks();

  arduboy.display();
}
