#pragma once

#include <stdint.h>

#if defined(ARDUINO_ARCH_AVR)

#define finline __attribute__((always_inline)) inline

namespace std
{
	template<typename T>
	finline void swap(T& a, T& b) { auto t = a; a = b; b = t; }

	template<class T, size_t N>
	class array
	{
	public:
		using value_type = T;
		using size_type = size_t;
		using difference_type = ptrdiff_t;
		using reference = value_type&;
		using const_reference = const value_type&;

	public:
		finline reference at(size_type pos) { return _data[pos]; }
		finline const_reference constexpr at(size_type pos) const { return _data[pos]; }
		finline reference operator[](size_type pos) { return _data[pos]; }
		finline const_reference constexpr operator[](size_type pos) const { return _data[pos]; }
		finline reference front() { return _data[0]; }
		finline const_reference constexpr front() const { return _data[0]; }
		finline reference back() { return _data[N-1]; }
		finline const_reference constexpr back() const { return _data[N-1]; }
		finline T* data() { return _data; }
		finline T const* data() const { return _data; }

		finline bool constexpr empty() const { return false; }
		finline size_type constexpr size() const { return N; }
		finline size_type constexpr max_size() const { return N; }

		void fill(T const& value)
		{
			for(auto n = size_type{0}; n < N; ++n)
				_data[n] = value;
		}
	private:
		T _data[N];
	};
}

#else

#include <cstring>
#include <array>

#define finline inline
#define PROGMEM
#define _BV(bit) (1<<(bit))

inline uint8_t pgm_read_byte(uint8_t const* p) { return *p; }
template<typename T>
inline uint16_t pgm_read_word(T const* p) { return *reinterpret_cast<const uint16_t*>(p); }

#endif

namespace z
{
	template<class... T>
	void unused(T&&...) {}
}

