// Zoo at Midnight
// Version 1.0
// Last Edited: 15/6/17
// (c) Luke Carter (2017)
// (@Hundstrasse www.Hundstrasse.com)
// For more info visit the website

//Should work fine with original Arduboy library, but untested
#include <Arduboy2.h>
Arduboy2 arduboy;

#include "Zoo_data.h"
#include "Menu.h"
Menu menu;
#include "Level.h"
Level level ;
#include "Special.h"
Special special;
#include "Scrawf.h"
Scrawf scrawf;
#include "Guard.h"

//Initialise guards
Guard Terry;
Guard Dean;
Guard Dan;
Guard Geoff;

//Global game variables
byte gametime = 0;
byte mapref = 0;
bool button_flag = false;

//Start Fresh game
void newgame () {
  mapref = 2;       //starting tile
  gametime = 0;
  level.Grab_map (mapref);
  Terry.Grab_guard (mapref, 0);
  Dean.Grab_guard (mapref, 1);
  Dan.Grab_guard (mapref, 2);
  Geoff.Grab_guard (mapref, 3);
  scrawf.Set_pos (59, 32);
  special.Reset ();
}

void intro_seq () { // Hundstrasse Logo intro
  arduboy.clear();
  arduboy.display();
  delay (100);
  for (int k = 0; k < 3; k++) {
    arduboy.drawSlowXYBitmap(0, 0, hundstrasse, WIDTH, HEIGHT, 1);
    arduboy.setCursor(32, 54);
    arduboy.print("@Hundstrasse");
    for (int i = 0; i < 8; i ++) {
      for (int j = 0; j < 16; j ++) {
        arduboy.fillRect(((j * 8) + 1 + k), ((i * 8) + 1 + k), (6 - (k * 2)), (6 - (k * 2)) , 0);
      }
    }
    arduboy.display();
    delay (50);
  }
  arduboy.drawSlowXYBitmap(0, 0, hundstrasse, WIDTH, HEIGHT, 1);
  arduboy.setCursor(32, 54);
  arduboy.print("@Hundstrasse");
  arduboy.display();
  delay (1500);
}

void alert_anim () {
  //Animation for alert status
  //Probably should have been part of the Guard class

  arduboy.display();

  for (int i = 0; i < 7; i++) {
    arduboy.drawSlowXYBitmap(15, 15, alert, 96, 32, 1);
    arduboy.fillRect (15, (15 + (i * 2)), 96, (32 - (i * 4)), 0);
    while (!(arduboy.nextFrame()))
    {}

    arduboy.display();
  }
  arduboy.drawSlowXYBitmap(15, 15, alert, 96, 32, 1);
  while (!(arduboy.nextFrame()))
  {}

  arduboy.display();
  delay (500);

  //Reset tile following alert
  gametime = 0;
  Terry.Grab_guard (mapref, 0);
  Dean.Grab_guard (mapref, 1);
  Dan.Grab_guard (mapref, 2);
  Geoff.Grab_guard (mapref, 3);

  //Yup, Game Over if you're playing hard
  if (menu.Hard()) {
    arduboy.clear();
    arduboy.setCursor(28, 28 );
    arduboy.print(" Game Over");
    arduboy.display();
    delay (1000);
    menu.Endgame();
  }

}

//Single Frame in Game
void gamestep () {

  // Loop Gametime
  if (gametime > 15) gametime = 0;

  //Move Scrawf & Change maptile
  byte oldmapref = mapref;
  mapref = scrawf.Move(mapref);
  if (mapref != oldmapref) {
    gametime = 0;
    Terry.Grab_guard (mapref, 0);
    Dean.Grab_guard (mapref, 1);
    Dan.Grab_guard (mapref, 2);
    Geoff.Grab_guard (mapref, 3);
    level.Grab_map (mapref);
  }

  // Move Guards
  Terry.Move (gametime);
  Dean.Move (gametime);
  Dan.Move (gametime);
  Geoff.Move(gametime);

  //check for inventory
  if ( menu.Ingame()) {
    if (!button_flag) button_flag = special.Inventory ();
    if (!(arduboy.pressed(A_BUTTON) || arduboy.pressed(B_BUTTON))) button_flag = false;

    arduboy.clear();

    // Draw level
    level.Draw ();

    //Draw Scrawf
    scrawf.Draw();

    // Draw Guards
    Terry.Draw (gametime);
    Dean.Draw (gametime);
    Dan.Draw (gametime);
    Geoff.Draw (gametime);

    //Draw Special Graphics
    special.Draw (mapref, gametime);
  }
  // Advance Gametime
  gametime ++;

  //Check for alert and do something
  if (scrawf.Check_alert()) {
    alert_anim ();
    scrawf.Reset ();
  }
}

void setup() {
  arduboy.begin();
  arduboy.setFrameRate(60);
  arduboy.clear();
  arduboy.display();
  intro_seq ();

}

void loop() {
  //Main game loop which I managed to keep quite clean

  if (!(arduboy.nextFrame())) return;

  if (menu.Ingame ()) {
    gamestep ();
  }
  else {
    menu.Check ();
    if (! menu.Ingame ()) menu.Draw ();
    if (menu.Ingame ()) newgame ();
  }

  arduboy.display();
}
